@@ -40,7 +40,7 @@ def simplify_vars(
                 i += 1
             new_vars[stripped_key] = val
             new_query_str = re.sub(
-                rf"(\$)({key})(\W)", r"\1" + stripped_key + r"\3", new_query_str
+                rf"(\$)({key})(\W*)", r"\1" + stripped_key + r"\3", new_query_str
             )
     if new_query_str[-1] == ";":
         new_query_str = new_query_str[0:-1]
