@@ -133,8 +133,9 @@ async def {link.name}(
 def build_resolver_link_function_str(node_resolver_name: str, link: Link) -> str:
     link_resolver_name = f"{link.target.model_name}Resolver"
     return f"""
-def {link.name}(self, _: T.Optional[{link_resolver_name}] = None, /) -> {node_resolver_name}:
-    self._nested_resolvers.add("{link.name}", _ or {link_resolver_name}())
+def {link.name}(self, _: T.Optional[{link_resolver_name}] = None, /, upsert: bool = False) -> {node_resolver_name}:
+    f = self._nested_resolvers.add if not upsert else self._nested_resolvers.upsert
+    f("{link.name}", _ or {link_resolver_name}())
     return self
     """
 
