@@ -6,11 +6,11 @@ from zoneinfo import ZoneInfo
 import json
 from fastapi.encoders import jsonable_encoder
 import edgedb
-from devtools import debug
+from devtools import debug as dev_debug
 from pydantic import BaseModel, PrivateAttr, Field
 
-from .unset import is_unset
-from .cache import CacheManager
+from .unset import UNSET
+from .cache import Cache
 from .execute import query as execute_query
 from .batch import Batch
 from .constants import random_str
@@ -154,53 +154,10 @@ def basemodel_to_str(
     )
 
 
-def resolver_equal_or_better(
-    resolver_a: "ResolverType", resolver_b: "ResolverType"
-) -> bool:
-    # TODO must compare variables as well
-    a_nested_vars = resolver_a._get_nested_query_variables()
-    b_nested_vars = resolver_b._get_nested_query_variables()
-    if a_nested_vars != b_nested_vars:
-        # if b has any vars a does not have
-        if len(set(b_nested_vars.keys()) - set(a_nested_vars.keys())) > 0:
-            return False
-        for k, v in a_nested_vars.items():
-            if k not in b_nested_vars:
-                # continue because this will be picked up in the filters if it is important
-                continue
-            if b_nested_vars[k] != v:
-                print(
-                    f"{k=}, {b_nested_vars[k]=} is not equal to {v=} so returning false"
-                )
-                return False
-
-    if resolver_a.to_str() == resolver_b.to_str():
-        return True
-    if resolver_a.all_filters_str() != resolver_b.all_filters_str():
-        return False
-    # check if rez b has appendix fields or other fields to return that A does not
-    if len(resolver_b.get_fields_to_return() - resolver_a.get_fields_to_return()) > 0:
-        return False
-    # now go to nesteds
-    for (
-        nested_resolver_name_b,
-        nested_resolver_b,
-    ) in resolver_b._nested_resolvers.items():
-        nested_resolver_a = resolver_a._nested_resolvers.get(nested_resolver_name_b)
-        if not nested_resolver_a:
-            print(
-                f"{nested_resolver_name_b} is not in the cached {resolver_a._model_name} resolver"
-            )
-            return False
-        if not resolver_equal_or_better(nested_resolver_a, nested_resolver_b):
-            return False
-    return True
-
-
 class Node(BaseModel, T.Generic[InsertType, PatchType]):
     id: UUID = Field(..., allow_mutation=False)
 
-    _cache: CacheManager = PrivateAttr(default_factory=CacheManager)
+    _cache: Cache = PrivateAttr(default_factory=Cache)
     _used_resolver: "ResolverType" = PrivateAttr(None)
     _original_dict: dict = PrivateAttr(None)
     _deleted: bool = PrivateAttr(None)
@@ -274,7 +231,7 @@ class Node(BaseModel, T.Generic[InsertType, PatchType]):
         )
 
     @property
-    def cache(self) -> CacheManager:
+    def cache(self) -> Cache:
         return self._cache
 
     @classmethod
@@ -294,30 +251,26 @@ class Node(BaseModel, T.Generic[InsertType, PatchType]):
         edge_name: str,
         edge_resolver: "TempResolverType",
         refresh: bool = False,
-        force_use_stale: bool = False,
-        print_forced_refresh: bool = True,
+        revert_to_first: bool = False,
+        allow_fetching: bool = True,
+        debug: bool = False,
     ) -> T.Optional[T.Union[TempNodeType, T.List[TempNodeType]]]:
-        """return await self.resolve(
-            name="artist", resolver=resolver, refresh=refresh, use_stale=use_stale
-        )
-        DID NOT INCLUDE STRICT, maybe will need it but for now i do not"""
         if refresh:
-            self.cache.remove(edge_name)
-        if edge_cache := self.cache.get(edge_name):
-            if force_use_stale:
-                return edge_cache.val
-            if not resolver_equal_or_better(edge_cache.resolver, edge_resolver):
-                if print_forced_refresh:
-                    print(
-                        f"resolvers are different so removing {edge_name} from the cache, old: {edge_cache.resolver.to_str()}, new: {edge_resolver.to_str()}"
-                    )
-                self.cache.remove(edge_name)
-        if not self.cache.exists(edge_name):
-            self.used_resolver._nested_resolvers[edge_name] = edge_resolver
-            self._used_resolver.clear_top_level_filters_and_variables()
-            obj = await self.used_resolver.get(id=self.id)
-            self.cache.replace(key=edge_name, cache=obj.cache.get(edge_name))
-        return self.cache.get_val(edge_name)
+            # FIXME only remove specific resolver? But what about subsets
+            self.cache.clear(edge=edge_name)
+        val = self.cache.val(
+            edge=edge_name, resolver=edge_resolver, revert_to_first=revert_to_first
+        )
+        if val is not UNSET:
+            return val
+        # if unset, must get this
+        if not allow_fetching:
+            raise NodeException(f"Not allowed to fetch edge {edge_name}.")
+        self.used_resolver._nested_resolvers.add(edge=edge_name, resolver=edge_resolver)
+        self._used_resolver.clear_top_level_filters_and_variables()
+        obj: Node = await self.used_resolver.gerror(id=self.id)
+        self.cache.replace(edge=edge_name, cache_nodes=obj.cache.get(edge_name))
+        return self.cache.val(edge=edge_name, resolver=edge_resolver)
 
     """CRUDS"""
 
@@ -616,7 +569,7 @@ for item in json_array_unpack(raw_data) union ({insert_str}) {given_resolver.to_
         update_d = {}
         for field in patch.__fields__.keys():
             val = getattr(patch, field)
-            if not is_unset(val):
+            if val is not UNSET:
                 if val is None:
                     val = set()
                 elif isinstance(val, BaseModel):
@@ -737,7 +690,7 @@ for item in json_array_unpack(raw_data) union ({insert_str}) {given_resolver.to_
         given_client: edgedb.AsyncIOClient = None,
     ) -> T.Optional[T.Union[ThisNodeType, T.List[ThisNodeType]]]:
         if update_last_updated:
-            if is_unset(getattr(patch, "last_updated", None)):
+            if getattr(patch, "last_updated", None) is UNSET:
                 patch.last_updated = datetime.now(tz=ZoneInfo("America/New_York"))
         update_d = cls.build_update_d_from_patch(patch=patch)
         set_links_d = make_set_links_d(model=patch, model_d=update_d)
