@@ -111,6 +111,10 @@ class Resolver(BaseModel, T.Generic[NodeType]):
     def set_field_dependency_map(cls, field_dep_map: FieldDepMapAlias) -> None:
         cls._field_dependency_map = field_dep_map
 
+    @property
+    def nested_resolvers(self) -> NestedResolvers:
+        return self._nested_resolvers
+
     @classmethod
     @property
     def field_dependency_map(cls) -> FieldDepMapAlias:
